/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		Config.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	05.08.2001
// letzte nderung:	07.11.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Diese Klasse hlt alle aktuellen Konfigurationsdaten. 			
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "Config.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::Config
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Konstruktor, initialisiert alle Member der Klasse
//			testet wieviel Level neu verfgbar sind in den MultiplayerModes
/////////////////////////////////////////////////////////////////////////////////////
Config::Config(void)
{
	Field fieldTmp;

	//wieviel neue Level bei DM?
	m_iActualLevel[1]=iNumberOfDeathMatchLevel;
	m_gmActual=DeathMatch;
	//erhhen, solange Laden mglich
	do
	{
		m_iActualLevel[1]++;
		iNumberOfDeathMatchLevel++;
	} while (loadLevel(&fieldTmp));
	// der letzte ging nicht mehr
	iNumberOfDeathMatchLevel--;

	//wieviel neue Level bei CTF?
	m_iActualLevel[3]=iNumberOfCTFLevel;
	m_gmActual=CaptureTheFlag;
	//erhhen, solange Laden mglich
	do
	{
		m_iActualLevel[3]++;
		iNumberOfCTFLevel++;
	} while (loadLevel(&fieldTmp));
	// der letzte ging nicht mehr
	iNumberOfCTFLevel--;

	//wieviel neue Level bei EEA?
	m_iActualLevel[4]=iNumberOfEatEmAllLevel;
	m_gmActual=EatEmAll;
	//erhhen, solange Laden mglich
	do
	{
		m_iActualLevel[4]++;
		iNumberOfEatEmAllLevel++;
	} while (loadLevel(&fieldTmp));
	// der letzte ging nicht mehr
	iNumberOfEatEmAllLevel--;

	//Konfiguration laden
	load();
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::~Config
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Destruktor
/////////////////////////////////////////////////////////////////////////////////////
Config::~Config()
{
	save();
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::load
/////////////////////////////////////////////////////////////////////////////////////
// Info:	versucht die Config-Datei einzulesen
/////////////////////////////////////////////////////////////////////////////////////
void Config::load()
{
	int iTmp;

	XConfigFile xcfRead("snakez.cfg");

	//Player 1 keys
	xcfRead.getAsInt("KEY_00",m_iKeyIds[0][0],DIK_UP);//up
	xcfRead.getAsInt("KEY_01",m_iKeyIds[0][1],DIK_DOWN);//down
	xcfRead.getAsInt("KEY_02",m_iKeyIds[0][2],DIK_LEFT);//left
	xcfRead.getAsInt("KEY_03",m_iKeyIds[0][3],DIK_RIGHT);//right
	xcfRead.getAsInt("KEY_04",m_iKeyIds[0][4],DIK_RSHIFT);//turn ccw
	xcfRead.getAsInt("KEY_05",m_iKeyIds[0][5],DIK_RCONTROL);//turn cw

	//Player 2 keys
	xcfRead.getAsInt("KEY_10",m_iKeyIds[1][0],DIK_W);//up
	xcfRead.getAsInt("KEY_11",m_iKeyIds[1][1],DIK_S);//down
	xcfRead.getAsInt("KEY_12",m_iKeyIds[1][2],DIK_A);//left
	xcfRead.getAsInt("KEY_13",m_iKeyIds[1][3],DIK_D);//right
	xcfRead.getAsInt("KEY_14",m_iKeyIds[1][4],DIK_LSHIFT);//turn ccw
	xcfRead.getAsInt("KEY_15",m_iKeyIds[1][5],DIK_LCONTROL);//turn cw

	//Player 3 keys
	xcfRead.getAsInt("KEY_20",m_iKeyIds[2][0],DIK_I);//up
	xcfRead.getAsInt("KEY_21",m_iKeyIds[2][1],DIK_K);//down
	xcfRead.getAsInt("KEY_22",m_iKeyIds[2][2],DIK_J);//left
	xcfRead.getAsInt("KEY_23",m_iKeyIds[2][3],DIK_L);//right
	xcfRead.getAsInt("KEY_24",m_iKeyIds[2][4],DIK_Y);//turn ccw
	xcfRead.getAsInt("KEY_25",m_iKeyIds[2][5],DIK_H);//turn cw

	//Defaultgeschwindigkeit
	xcfRead.getAsInt("Speed",m_iGameSpeed,125);
	
	// gewhlter Level
	xcfRead.getAsInt("Level_0",m_iActualLevel[0],1);
	xcfRead.getAsInt("Level_1",m_iActualLevel[1],1);
	xcfRead.getAsInt("Level_2",m_iActualLevel[2],1);
	xcfRead.getAsInt("Level_3",m_iActualLevel[3],1);
	xcfRead.getAsInt("Level_4",m_iActualLevel[4],1);
	
	//DefaultMode ist SinglePlayer
	xcfRead.getAsInt("GameMode",iTmp,(int)SinglePlay);
	m_gmActual=(GameMode)iTmp;

	//active Spieler
	xcfRead.getAsInt("Players",m_iNrOfActivePlayers,1);

	//nur die erste Map ist freigeschalten
	xcfRead.getAsInt("MapsFree",iTmp,0);

	if (	(iTmp & 0xff)-125 == (((iTmp&0xff00)/0x100)^112)
		&&  (iTmp & 0xff)+42 == (((iTmp&0xff0000)/0x10000)^58) )
	{
		m_iMapsFree=(iTmp & 0xff)-137;
	}
	else
	{
		m_iMapsFree=1;
		m_iActualLevel[0]=1;
	}

	//Map geht am Rand nicht weiter
	xcfRead.getAsBool("WrapAround",m_bWrapAround,true);
	//welchesTile ist aussen?
	xcfRead.getAsInt("WrapAroundTile",m_iWrapAroundTile,64);

	//SFX volle Lautstrke
	xcfRead.getAsInt("SFXVolume",m_iSFXVolume,23);

	//Musik 1/2 Lautstrke
	xcfRead.getAsInt("MusicVolume",m_iMusicVolume,12);

	//Partikel werden gezeigt
	xcfRead.getAsBool("Particles",m_bParticles,true);
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::save
/////////////////////////////////////////////////////////////////////////////////////
// Info:		versucht die Config-Datei abzuspeichern
/////////////////////////////////////////////////////////////////////////////////////
void Config::save()
{
	int iCount1;
	int iCount2;

	FILE* fileOutput;

	fileOutput=fopen ("snakez.cfg","wt");

		if (fileOutput==NULL)
	{
		XLogging::log("Config::save > (FEHLER!!!) Speichervorgang gescheitert !!!\n");
		return;//Speichern gescheitert 
	}

	fprintf(fileOutput,"[Keys]\n\n");

	for (iCount1=0;iCount1<iNumberOfPlayers;iCount1++)
		for (iCount2=0;iCount2<iNrOfKeys;iCount2++)
		{
			fprintf(	fileOutput,
						"KEY_%1d%1d = %d\n",
						iCount1,
						iCount2,
						m_iKeyIds[iCount1][iCount2]);
		}

		
	fprintf(fileOutput,"\n[Game]\n\n");

	for (iCount1=0;iCount1<5;iCount1++)
		{
			fprintf(	fileOutput,
						"Level_%1d = %d\n",
						iCount1,
						m_iActualLevel[iCount1]);
		}

	fprintf(	fileOutput,"Speed = %d\n",m_iGameSpeed);
	fprintf(	fileOutput,"GameMode = %d\n",(int)m_gmActual);
	fprintf(	fileOutput,"Players = %d\n",m_iNrOfActivePlayers);

	iCount1=	(m_iMapsFree + 137) & 0xff 
			|   (((m_iMapsFree+12) ^ 112) & 0xff) *0x100
			|   (((m_iMapsFree+179) ^ 58) & 0xff) *0x10000;
	fprintf(	fileOutput,"MapsFree = %d\n",iCount1);

	if (m_bWrapAround)
		fprintf(	fileOutput,"WrapAround = true\n");
	else
		fprintf(	fileOutput,"WrapAround = false\n");

	fprintf(	fileOutput,"WrapAroundTile = %d\n",m_iWrapAroundTile);

	fprintf(fileOutput,"\n[Audio&Video]\n\n");

	fprintf(	fileOutput,"SFXVolume = %d\n",m_iSFXVolume);
	fprintf(	fileOutput,"MusicVolume = %d\n",m_iMusicVolume);

	if (m_bParticles)
		fprintf(	fileOutput,"Particles = true\n");
	else
		fprintf(	fileOutput,"Particles = false\n");

	fclose(fileOutput);
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::setDefault
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt die Default Konfiguration
//			seit Benutzung der ConfigKlasse ist diese Methode berflssig
/////////////////////////////////////////////////////////////////////////////////////
/*
void Config::setDefault()
{
	//Player 1 keys
	m_iKeyIds[0][0]=DIK_UP;//up
	m_iKeyIds[0][1]=DIK_DOWN;//down
	m_iKeyIds[0][2]=DIK_LEFT;//left
	m_iKeyIds[0][3]=DIK_RIGHT;//right
	m_iKeyIds[0][4]=DIK_RSHIFT;//turn ccw
	m_iKeyIds[0][5]=DIK_RCONTROL;//turn cw

	//Player 2 keys
	m_iKeyIds[1][0]=DIK_W;//up
	m_iKeyIds[1][1]=DIK_S;//down
	m_iKeyIds[1][2]=DIK_A;//left
	m_iKeyIds[1][3]=DIK_D;//right
	m_iKeyIds[1][4]=DIK_LSHIFT;//turn ccw
	m_iKeyIds[1][5]=DIK_LCONTROL;//turn cw

	//Player 3 keys
	m_iKeyIds[2][0]=DIK_I;//up
	m_iKeyIds[2][1]=DIK_K;//down
	m_iKeyIds[2][2]=DIK_J;//left
	m_iKeyIds[2][3]=DIK_L;//right
	m_iKeyIds[2][4]=DIK_Y;//turn ccw
	m_iKeyIds[2][5]=DIK_H;//turn cw

	//Defaultgeschwindigkeit
	m_iGameSpeed=125;
	
	m_iActualLevel[0]=1;
	m_iActualLevel[1]=1;
	m_iActualLevel[2]=1;
	m_iActualLevel[3]=1;
	m_iActualLevel[4]=1;
	
	//DefaultMode ist SinglePlayer
	m_gmActual=SinglePlay;
	m_iNrOfActivePlayers=1;

	//nur die erste Map ist freigeschalten
	m_iMapsFree=1;
	/////////////////
	//zum Testen zwei
	m_iMapsFree=2;

	//Map geht am Rand nicht weiter
	m_bWrapAround=false;
	//welchesTile ist aussen?
	m_iWrapAroundTile=64;

	// zu Testzwecken!!! unvollstndiges load aufrufen
	load();

	//speichern
	save();
}
*/


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::getKey
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt die DI-KeyNummer, der nten Taste vom soundsovielten Spieler
/////////////////////////////////////////////////////////////////////////////////////
int Config::getKey(int iPlayer, int iNr)
{
	if (iPlayer>=0 && iNr>=0 && iPlayer<iNumberOfPlayers && iNr<iNrOfKeys)
	{
		return m_iKeyIds[iPlayer][iNr];
	}
	else
	{
		return -1;
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::setKey
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt die entprechende Taste
/////////////////////////////////////////////////////////////////////////////////////
bool Config::setKey(int iPlayer, int iNr, int iKeyId)
{
	if (iPlayer>=0 && iNr>=0 && 
		iPlayer<iNumberOfPlayers && iNr<iNrOfKeys &&
		iKeyId>=0 && iKeyId<256)
	{
		m_iKeyIds[iPlayer][iNr]=iKeyId;
		return true;
	}
	else
	{
		return false;
	}
	
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::loadLevel
/////////////////////////////////////////////////////////////////////////////////////
// Info:	konstruiert ein FileName und ld das Level in das Field
/////////////////////////////////////////////////////////////////////////////////////
bool Config::loadLevel(Field* pfieldDest)
{
	bool bLoadSuccess;
	int iCount;

	char pcName[20]="level\\sn";
	char pcNumber[3];

	//je nach Spielmodus PreFix anhngen
	switch(m_gmActual)
	{
	case SinglePlay:
		strcat(pcName,"map");
		break;
	case DeathMatch:
		strcat(pcName,"dm");
		break;
	case Challenge:
		strcat(pcName,"ch");
		break;
	case CaptureTheFlag:
		strcat(pcName,"ctf");
		break;
	case EatEmAll:
		strcat(pcName,"eea");
		break;
	}

	//Levelnr in String schreiben
	sprintf(pcNumber,"%02d",getLevelNr());
	strcat(pcName,pcNumber);

	//extension anhngen
	strcat(pcName,".lvl");
	
	//Level ins field laden
	bLoadSuccess=pfieldDest->load(pcName);

	//Umrandung machen, wenn eine da sein soll
	if (bLoadSuccess && m_bWrapAround)
	{
		for (iCount=0;iCount<iFieldSizeX;iCount++)
		{
			pfieldDest->setTile(iCount,0,m_iWrapAroundTile,true);
			pfieldDest->setTile(iCount,iFieldSizeY-1,m_iWrapAroundTile,true);
		}

		for (iCount=1;iCount<iFieldSizeY-1;iCount++)
		{
			pfieldDest->setTile(0,iCount,m_iWrapAroundTile,true);
			pfieldDest->setTile(iFieldSizeX-1,iCount,m_iWrapAroundTile,true);
		}
	}

	return bLoadSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Config::getLevelNr
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt den aktuellen Level zurueck, mit Delta kann die LevelNummer
//			verndert werden (z.b. Delta=1 => einen Level hoch schalten)
/////////////////////////////////////////////////////////////////////////////////////
int Config::getLevelNr(int iDelta/* =0 */)
{
	switch (m_gmActual)
	{
	case SinglePlay:
		m_iActualLevel[0]+=iDelta;
		if (m_iActualLevel[0]<1)
			m_iActualLevel[0]=1;
		if (m_iActualLevel[0]>iNumberOfSinglePlayLevel)
			m_iActualLevel[0]=iNumberOfSinglePlayLevel;
		return m_iActualLevel[0];
	case DeathMatch:
		m_iActualLevel[1]+=iDelta;
		if (m_iActualLevel[1]<1)
			m_iActualLevel[1]=1;
		if (m_iActualLevel[1]>iNumberOfDeathMatchLevel)
			m_iActualLevel[1]=iNumberOfDeathMatchLevel;
		return m_iActualLevel[1];
	case Challenge:
		m_iActualLevel[2]+=iDelta;
		if (m_iActualLevel[2]<1)
			m_iActualLevel[2]=1;
		if (m_iActualLevel[2]>iNumberOfChallengeLevel)
			m_iActualLevel[2]=iNumberOfChallengeLevel;
		return m_iActualLevel[2];
	case CaptureTheFlag:
		m_iActualLevel[3]+=iDelta;
		if (m_iActualLevel[3]<1)
			m_iActualLevel[3]=1;
		if (m_iActualLevel[3]>iNumberOfCTFLevel)
			m_iActualLevel[3]=iNumberOfCTFLevel;
		return m_iActualLevel[3];
	case EatEmAll:
		m_iActualLevel[4]+=iDelta;
		if (m_iActualLevel[4]<1)
			m_iActualLevel[4]=1;
		if (m_iActualLevel[4]>iNumberOfEatEmAllLevel)
			m_iActualLevel[4]=iNumberOfEatEmAllLevel;
		return m_iActualLevel[4];
	}
	
	return -1;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	Config.cpp
/////////////////////////////////////////////////////////////////////////////////////
