/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		HighScore.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	19.09.2001
// letzte nderung:	21.09.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Die HighScore Klasse kann eine beliebige Anzahl von HighScore-Listen
//			verwalten. So ist es mglich mehrere HighScores in einer Datei zu
//			speichern.
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "HighScore.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::HighScore
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Konstruktor, initialisiert alle Member der Klasse
//			holt Speicher fr die Anzahl der Listen
/////////////////////////////////////////////////////////////////////////////////////
HighScore::HighScore(int iNrOfLists)
{
	m_iNrOfLists=iNrOfLists;

	m_phslData=new HighScoreList[m_iNrOfLists];

	if (m_phslData==NULL) //kein Speicher um Listen anzulegen
	{
		XLogging::log("HighScore::HighScore > (FEHLER!!!) Zuwenig Speicher !!!\n");
		exit(666);//verdammt
	}

	resetData();

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::~HighScore
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Destruktor, gibt den Speicher frei
/////////////////////////////////////////////////////////////////////////////////////
HighScore::~HighScore(void)
{
	delete[] m_phslData;
	m_phslData=NULL;

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::resetData
/////////////////////////////////////////////////////////////////////////////////////
// Info:	fllt die Listen mit Nullen und LeerStrings
/////////////////////////////////////////////////////////////////////////////////////
void HighScore::resetData()
{
	int iList;
	int iEntry;

	for(iList=0;iList<m_iNrOfLists;iList++)
		for (iEntry=0;iEntry<iHighScoreEntries;iEntry++)
		{
			m_phslData[iList].iPoints[iEntry]=0;
			m_phslData[iList].pcNames[iEntry*(iHighScoreNameLength+1)]=0;
		}
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::load
/////////////////////////////////////////////////////////////////////////////////////
// Info:	ldt die HighScore-Daten von der HD, Rckgabe von true bei Erfolg
/////////////////////////////////////////////////////////////////////////////////////
bool HighScore::load(char* pcFileName)
{
	fstream fsInput;

	fsInput.open(pcFileName,ios::in | ios::binary| ios::nocreate);

	if (fsInput.fail())
	{ // Datei nicht gefunden
		resetData();
		return false;
	}
	
	// Daten lesen
	fsInput.read((char*)m_phslData,sizeof(HighScoreList)*m_iNrOfLists);

	if (fsInput.eof())
	{ // Dateilnge fehlerhaft
		fsInput.close();
		resetData();
		return false;
	}

	fsInput.close();
		
	return true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::save
/////////////////////////////////////////////////////////////////////////////////////
// Info:	speichert die Daten auf HD, Rckgabe von true bei Erfolg
/////////////////////////////////////////////////////////////////////////////////////
bool HighScore::save(char* pcFileName)
{
	fstream fsOutput;

	fsOutput.open(pcFileName,ios::out | ios::binary);

	if (fsOutput.fail())
	{ // Fehler beim Datei erzeugen
		return false;
	}
	
	// Daten schreiben
	fsOutput.write((char*)m_phslData,sizeof(HighScoreList)*m_iNrOfLists);

	fsOutput.close();
		
	return true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::getEntry
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt den nten Eintrag der nten Liste
/////////////////////////////////////////////////////////////////////////////////////
int HighScore::getEntry(int iList,int iEntry,char* pcDest)
{
	if(	iList<0 || iList>=m_iNrOfLists || 
		iEntry<0 || iEntry>=iHighScoreEntries)
	{
		strcpy(pcDest,"none");
		return -1;
	}

	strcpy(pcDest,(m_phslData[iList].pcNames)+iEntry*(iHighScoreNameLength+1));
	return m_phslData[iList].iPoints[iEntry];	
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::setEntry
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt einen Eintrag in der nten Liste
/////////////////////////////////////////////////////////////////////////////////////
bool HighScore::setEntry(int iList,char* pcSrc,int iPoints)
{
	int iPlace;
	int iTmp;

	//testen welcher Platz erreicht wurde
	iPlace=iHighScoreEntries;
	while (iPlace>0 && iPoints>m_phslData[iList].iPoints[iPlace-1])
	{
		iPlace--;
	}

	if (iPlace==iHighScoreEntries)
		return false;

	//eventuell nachfolgende Pltze verschieben
	iTmp=iHighScoreEntries-1;
	while(iTmp>iPlace)
	{
		strcpy(	(m_phslData[iList].pcNames)+iTmp*(iHighScoreNameLength+1),
				(m_phslData[iList].pcNames)+(iTmp-1)*(iHighScoreNameLength+1));
		m_phslData[iList].iPoints[iTmp]=m_phslData[iList].iPoints[iTmp-1];

		iTmp--;
	}

	strcpy(	(m_phslData[iList].pcNames)+iPlace*(iHighScoreNameLength+1),
			pcSrc);
	m_phslData[iList].iPoints[iPlace]=iPoints;

	return true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::getMinimumPoints
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt die Minimalzahl von Punkten in der nten Liste an
/////////////////////////////////////////////////////////////////////////////////////
int HighScore::getMinimumPoints(int iList)
{
	if(	iList<0 || iList>=m_iNrOfLists)
	{
		return -1;
	}
	
	return m_phslData[iList].iPoints[iHighScoreEntries-1];
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			HighScore::getNameLength
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt die Lnge der Namen in der HighScoreliste zurck
/////////////////////////////////////////////////////////////////////////////////////
int HighScore::getNameLength()
{
	return iHighScoreNameLength;	
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	HighScore.cpp
/////////////////////////////////////////////////////////////////////////////////////
