/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		Snake.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	05.08.2001
// letzte nderung:	26.09.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Die Klasse Snake enthlt eine verkettete Liste mit den Teilen einer
//			Schlange. Die Schlange empfngt Befehle wie "WEITERBEWEGEN" o.
//			"RICHTUNG NDERN". Die Richtungsnderungen werden auch in einer Queue
//			geparkt und bei der nchsten Bewegung ausgefhrt
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "Snake.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::Snake
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Konstruktor, initialisiert alle Member der Klasse
//			setzt die Schlange an die Koordinaten
//			Die Schlange verwendet die Tiles ab TileOffset
/////////////////////////////////////////////////////////////////////////////////////
Snake::Snake(int iStartX, int iStartY,int iDirection, int iTileOffset)
{
	m_iTileOffset=iTileOffset;

	m_iLength=0;

	m_iSoundFX=-1;
	
	reset(iStartX,iStartY,iDirection);
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::~Snake
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Destruktor, Speicher zurck...
/////////////////////////////////////////////////////////////////////////////////////
Snake::~Snake()
{
	releaseMemory();
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::reset
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt die Schlange an die Stelle x,y und die Lnge auf 2
/////////////////////////////////////////////////////////////////////////////////////
void Snake::reset(int iStartX, int iStartY,int iDirection)
{
	SnakePart* pspTmp;
	int iTmp;

	releaseMemory();

	pspTmp=new SnakePart;
	if (pspTmp==NULL)
	{
		XLogging::log("Snake::reset >> (FEHLER!!!) Zuwenig Speicher !!!\n");
		exit(666);//verdammt
	}

	pspTmp->m_iPosX=iStartX;
	pspTmp->m_iPosY=iStartY;
	pspTmp->m_pspNext=NULL;
	pspTmp->m_pspPrev=NULL;

	m_pspFirst=pspTmp;
	m_pspLast=pspTmp;

	m_iLength=1;
	m_iAdditionalLength=1;

	m_iDirection=iDirection;
	
	for(iTmp=1;iTmp<8;iTmp++)
		m_iDirQueue[iTmp]=4;
	
	m_iDirQueue[0]=iDirection;
	m_iQueueWritePos=1;
	m_iQueueReadPos=1;

	m_bAlive=true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::releaseMemory
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt alle Teile und den davon belegten Speicher zurck
//			(das Lschen der Tiles aus der TileMap ist dann nicht mehr mglich!)
/////////////////////////////////////////////////////////////////////////////////////
void Snake::releaseMemory()
{
	SnakePart* pspTmp;

	while (m_iLength>0)
	{
		pspTmp=m_pspLast;
		m_pspLast=m_pspLast->m_pspPrev;
		delete pspTmp;
		m_iLength--;
	}

	m_pspFirst=NULL;

	kill();
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::move
/////////////////////////////////////////////////////////////////////////////////////
// Info:	bewegt die Schlange weiter, fhrt dabei eventuelle Richtungsnderung aus
//			der Queue durch, Die Tilemap im Field wird entsprechend gendert
//			Rckgabewert ist der Wert des Feldes, den die Schlange betritt
/////////////////////////////////////////////////////////////////////////////////////
int Snake::move(Field* pfieldDest)
{
	int iTmp;
	int iOldTile;
	SnakePart* pspTmp;

	//TOT?
	if (!m_bAlive)
		return -1;

	m_iSoundFX=m_pspFirst->m_iPosX;

	//letztes SnakeTile in der Map frei geben, wenn nichts anzuhngen ist
	if (m_iAdditionalLength==0)
		pfieldDest->freeTile(m_pspLast->m_iPosX,m_pspLast->m_iPosY);

	//Richtung ndern
	iTmp=getFromQueue();
	if (iTmp!=4)
	{
		m_iDirection=iTmp;
		m_iSoundFX=m_pspFirst->m_iPosX+1024;
	}

	//neues SnakeTeil erzeugen
	pspTmp=new SnakePart;
	if (pspTmp==NULL)
	{
		XLogging::log("Snake::move >> (FEHLER!!!) Zuwenig Speicher !!!\n");
		exit(666);//das wr echt doof
	}

	//auf Position setzen
	pspTmp->m_iPosX=m_pspFirst->m_iPosX+getdeltaX(m_iDirection);
	pspTmp->m_iPosX=(pspTmp->m_iPosX+iFieldSizeX)%iFieldSizeX;
	pspTmp->m_iPosY=m_pspFirst->m_iPosY+getdeltaY(m_iDirection);
	pspTmp->m_iPosY=(pspTmp->m_iPosY+iFieldSizeY)%iFieldSizeY;

	//altes Tile sichern
	iOldTile=pfieldDest->getTile(pspTmp->m_iPosX,pspTmp->m_iPosY);

	//kein Vorgnger
	pspTmp->m_pspPrev=NULL;
	//Nachfolger ist das ehemals Erste
	pspTmp->m_pspNext=m_pspFirst;

	//das ehemals Erste hat nun einen Vorgnger
	m_pspFirst->m_pspPrev=pspTmp;

	//das Erste ist das jetzt erstellte
	m_pspFirst=pspTmp;

	if (m_iAdditionalLength>0)  //Teil anfgen?
	{
		m_iAdditionalLength--; //JA!
		m_iLength++;
	}
	else //wenn nicht letztes Teil lschen
	{
		pspTmp=m_pspLast->m_pspPrev;
		delete m_pspLast;
		m_pspLast=pspTmp;
		m_pspLast->m_pspNext=NULL;
	}

	//zwei ersten und zwei letzten Teile neu setzen
	putPart(m_pspFirst,pfieldDest);
	putPart(m_pspFirst->m_pspNext,pfieldDest);
	putPart(m_pspLast,pfieldDest);
	putPart(m_pspLast->m_pspPrev,pfieldDest);

	return iOldTile;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::emit
/////////////////////////////////////////////////////////////////////////////////////
// Info:	die Schlange emitiert Partikel des gewnschten Typs
//			sollte Impulse true sein, wird die Bewegung der Schlange beachtet
/////////////////////////////////////////////////////////////////////////////////////
void Snake::emit(Field* pfieldDest,
				 int iType, 
				 int iNrOfParticles, 
				 bool bImpulse/* =false */)
{
	int iTmp;
	float fImpulseX=0.0f;
	float fImpulseY=0.0f;

	if (bImpulse)
	{
		fImpulseX=(float)getdeltaX(m_iDirQueue[(m_iQueueWritePos+7)%8]);
		fImpulseY=(float)getdeltaY(m_iDirQueue[(m_iQueueWritePos+7)%8]);
	}

	for (iTmp=0;iTmp<iNrOfParticles;iTmp++)
	{
		pfieldDest->createParticle(	(float)m_pspFirst->m_iPosX,
									(float)m_pspFirst->m_iPosY,
									fImpulseX,
									fImpulseY,
									iType);
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::remove
/////////////////////////////////////////////////////////////////////////////////////
// Info:	entfernt alle Schlangenteile aus der Tilemap, beginnt am Ende der 
//			Schlange und endet eventl. an der angegebenen Position
//			der Rckgabewert entspricht der Anzahl der Tiles, die gelscht wurden
/////////////////////////////////////////////////////////////////////////////////////
int Snake::remove(Field* pfieldDest,int iPosX/* =-1 */, int iPosY/* =-1 */)
{
	int iPreviousLength=m_iLength+m_iAdditionalLength;

	m_iAdditionalLength=0;

	SnakePart* pspTmp;

	while (m_iLength>1)
	{
		pspTmp=m_pspLast;
		pfieldDest->freeTile(pspTmp->m_iPosX,pspTmp->m_iPosY);
		m_pspLast=m_pspLast->m_pspPrev;
		m_pspLast->m_pspNext=NULL;
		m_iLength--;
		if (pspTmp->m_iPosX==iPosX && pspTmp->m_iPosY==iPosY)
		{
			delete pspTmp;
			break;//aufhren zu lschen
		}
		delete pspTmp;
	}


	return (iPreviousLength-m_iLength);
	
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::removeUntilHead
/////////////////////////////////////////////////////////////////////////////////////
// Info:	entfernt von der anderen Schlange alle Teil bis zum Kopf dieser Schlange,
//			der Rckgabewert entspricht der Anzahl der Tiles, die gelscht wurden.	
/////////////////////////////////////////////////////////////////////////////////////
int Snake::removeUntilHead(Field* pfieldDest,Snake* psnakeDest)
{
	return psnakeDest->remove(	pfieldDest,
								m_pspFirst->m_iPosX,
								m_pspFirst->m_iPosY);
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::kill
/////////////////////////////////////////////////////////////////////////////////////
// Info:	"ttet" die Schlange ;)
//			ersetzt evtl den Kopf auf der Map mit dem angegebenen Tile
/////////////////////////////////////////////////////////////////////////////////////
void Snake::kill(Field* pfieldDest/* =NULL */, int iRemoveHeadWithTile/* =-1 */)
{
	if (iRemoveHeadWithTile>=0 && iRemoveHeadWithTile<256)
	{
		pfieldDest->setTile(	m_pspFirst->m_iPosX,
								m_pspFirst->m_iPosY,
								iRemoveHeadWithTile);
	}
	m_bAlive=false;	
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::alive
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt an ob die Schlange noch lebt oder schon kollidiert ist
/////////////////////////////////////////////////////////////////////////////////////
bool Snake::alive()
{
	return m_bAlive;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::add
/////////////////////////////////////////////////////////////////////////////////////
// Info:	verlngert die Schlange um eine bestimmte Lnge
/////////////////////////////////////////////////////////////////////////////////////
void Snake::add(int iSize)
{
	m_iAdditionalLength+=iSize;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::changeDirection
/////////////////////////////////////////////////////////////////////////////////////
// Info:	nimmt einen Richtungsbefehl an
//			-1 = drehen im Uhrzeigersinn
//			-2 = drehen gegen den Uhrzeigersinn
/////////////////////////////////////////////////////////////////////////////////////
void Snake::changeDirection(int iNewDirection)
{
	int iPreviousDirection;
	//vorhergehende Richtung holen
	iPreviousDirection=m_iDirQueue[(m_iQueueWritePos+7)%8];

	switch (iNewDirection)
	{
	case 0://Hoch
		if (iPreviousDirection%2 == 1)
			putInQueue(0);
		return;
	case 1://Rechts
		if (iPreviousDirection%2 == 0)
			putInQueue(1);
		return;
	case 2://Runter
		if (iPreviousDirection%2 == 1)
			putInQueue(2);
		return;
	case 3://Links
		if (iPreviousDirection%2 == 0)
			putInQueue(3);
		return;
	case -1://cw
		putInQueue((iPreviousDirection+5)%4);
		return;
	case -2://ccw
		putInQueue((iPreviousDirection+3)%4);
		return;
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::putPart
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt den Schlangenteil auf den Screen, nachdem berechnet wurde,
//			welches Teil es ist
/////////////////////////////////////////////////////////////////////////////////////
void Snake::putPart(SnakePart* pspSource, Field* pfieldDest)
{
	int iTile;
	int iPreviousDirection=4;
	int iNextDirection=4;

	if (pspSource!=NULL)
	{
		//calculate Tile
		//vorhergehende Richtung holen
		if (pspSource->m_pspNext!=NULL)
		{
			iPreviousDirection=
				getDirection(	pspSource->m_pspNext->m_iPosX-pspSource->m_iPosX,
								pspSource->m_pspNext->m_iPosY-pspSource->m_iPosY);
		}
		//nachfolgende Richtung holen
		if (pspSource->m_pspPrev!=NULL)
		{
			iNextDirection=
				getDirection(	pspSource->m_pspPrev->m_iPosX-pspSource->m_iPosX,
								pspSource->m_pspPrev->m_iPosY-pspSource->m_iPosY);
		}
		iTile=g_TileSetData.getSnakeTileOffset(iPreviousDirection,iNextDirection);
		//calculate Tile
		pfieldDest->setTile(	pspSource->m_iPosX,
								pspSource->m_iPosY,
								m_iTileOffset+iTile);
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::putInQueue
/////////////////////////////////////////////////////////////////////////////////////
// Info:	schmeisst einen Richtungsbefehl in die Queue, ist sie voll, wird er 
//			ignoriert
/////////////////////////////////////////////////////////////////////////////////////
void Snake::putInQueue(int iDirection)
{
	int iNewWritePos;

	iNewWritePos=(m_iQueueWritePos+1)%8;
	
	if (iNewWritePos==m_iQueueReadPos) return; //voll

	m_iDirQueue[m_iQueueWritePos]=iDirection;

	m_iQueueWritePos=iNewWritePos;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::getFromQueue
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt einen Richtungsbefehl aus der Queue, wenn keiner da ist, wird 4
//			zurckgegeben
/////////////////////////////////////////////////////////////////////////////////////
int Snake::getFromQueue()
{
	int iDirection=m_iDirQueue[m_iQueueReadPos];

	if (m_iQueueWritePos==m_iQueueReadPos) return 4; //leer

	m_iQueueReadPos=(m_iQueueReadPos+1)%8;

	return iDirection;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::getDirection
/////////////////////////////////////////////////////////////////////////////////////
// Info:	wandelt dx und dy in eine Richtung um, und das funktioniert so:
//				Richtung	dx	dy	Richtung als Zahl
//				HOCH		0	-1	0
//				RECHTS		1	0	1
//				RUNTER		0	1	2
//				LINKS		-1	0	3
//				Fehler		d	d	4
/////////////////////////////////////////////////////////////////////////////////////
int Snake::getDirection(int iDeltaX,int iDeltaY)
{
	if (iDeltaX==iFieldSizeX-1)
		iDeltaX=-1;
	if (iDeltaX==-iFieldSizeX+1)
		iDeltaX=1;

	if (iDeltaY==iFieldSizeY-1)
		iDeltaY=-1;
	if (iDeltaY==-iFieldSizeY+1)
		iDeltaY=1;

	switch(iDeltaY+iDeltaX*256)
	{
	case -1:
		return 0;
	case 256:
		return 1;
	case 1:
		return 2;
	case -256:
		return 3;
	}
	return 4;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::getdeltaX
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt dX bei der Richtung bla aus, funktioniert nach der Tabelle von oben
/////////////////////////////////////////////////////////////////////////////////////
int Snake::getdeltaX(int iDirection)
{
	switch (iDirection)	
	{
	case 1:
		return 1;
	case 3:
		return -1;
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::getdeltaY
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt dY bei der Richtung bla aus, funktioniert nach der Tabelle von oben
/////////////////////////////////////////////////////////////////////////////////////
int Snake::getdeltaY(int iDirection)
{
	switch (iDirection)	
	{
	case 0:
		return -1;
	case 2:
		return 1;
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Snake::getSoundFX
/////////////////////////////////////////////////////////////////////////////////////
// Info:	hiermit kann die Schlange SoundFX anfordern
//			-1 = kein Sound
//			0... = bewegt bei genau diesem x-wert
//			1024... = gedreht bei genau diesem x-wert
/////////////////////////////////////////////////////////////////////////////////////
int Snake::getSoundFX(void)
{
	int iTmp=m_iSoundFX;

	m_iSoundFX=-1;

	return iTmp;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	Snake.cpp
/////////////////////////////////////////////////////////////////////////////////////
