/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XInput.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Baal
//
// Version			1.1
// Erstelldatum:	04.09.2000
// letzte nderung:	29.07.2001 (by TGGC)
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Klasse, um Daten der Tastatur und der Maus mittels Direct-Input
//					abzufragen.
//					Daten werden ber den aktuellen Status des Gertes ermittelt
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000 by x-treme
/////////////////////////////////////////////////////////////////////////////////////


// XGame-Includes
#include "XInput.h"


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::XInput
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor
/////////////////////////////////////////////////////////////////////////////////////

XInput::XInput()
{
	m_lpDirectInput				= NULL;
	m_lpMouseDevice				= NULL; 
	m_lpKeyboardDevice			= NULL;
	m_lpJoystickDevice			= NULL;

	m_lpRect					= NULL;

	m_dwJoyXPos					=
	m_dwJoyYPos					= 0;

	m_bMouseButtonPressed[0]	=
	m_bMouseButtonPressed[1]	=
	m_bMouseButtonPressed[2]	= false;

	m_bJoyButtonPressed[0]		=
	m_bJoyButtonPressed[1]		=
	m_bJoyButtonPressed[2]		=
	m_bJoyButtonPressed[3]		= false;

	m_bJoyMoved[0]				=
	m_bJoyMoved[1]				=
	m_bJoyMoved[2]				=
	m_bJoyMoved[3]				= false;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::~XInput
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor
/////////////////////////////////////////////////////////////////////////////////////

XInput::~XInput()
{
	uninitialize();
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Initialisiert ein DirectInput Objekt
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::initialize(HINSTANCE hInstance)
{
	HRESULT hResult;

	// besteht Direct-Input Objekt schon?
	#if defined(XDEBUG) || defined(XINPUTDEBUG)
	if (m_lpDirectInput)
	{
		XLogging::log("XInput::initialize > (WARNUNG!!!) DirectInput7-Objekt existiert bereits\n");
		
		return DIERR_INVALIDPARAM;
	}
	#endif
	
	// DirectInput Objekt anlegen
	hResult = DirectInputCreateEx(hInstance, DIRECTINPUT_VERSION, IID_IDirectInput7, (void**)&m_lpDirectInput, NULL); 
	
	if(hResult != DI_OK) 
	{ 
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::initialize > (FEHLER!!!) Konnte DirectInput7-Objekt nicht erzeugen\n");
		#endif
		
		m_lpDirectInput = NULL;
		return hResult;
	}
	
	#if defined(XDEBUG) || defined(XINPUTDEBUG)
	XLogging::log("XInput::initialize > DirectInput-Objekt erzeugt\n");
	#endif

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::setJoystickProperties
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Initialisiert Eigenschaften des Joysticks
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::setJoystickProperties(int nRange, unsigned int nDeadZone)
{
	HRESULT hResult;

	DIPROPRANGE diPropRange;
	DIPROPDWORD diPropDword;
 
	// Daten setzen, um den Bereich der X-Achse zu definieren
	diPropRange.diph.dwSize       = sizeof(diPropRange); 
	diPropRange.diph.dwHeaderSize = sizeof(diPropRange.diph); 
	diPropRange.diph.dwObj        = DIJOFS_X; 
	diPropRange.diph.dwHow        = DIPH_BYOFFSET; 
	diPropRange.lMin              = -nRange; 
	diPropRange.lMax              = +nRange; 

	hResult = m_lpJoystickDevice->SetProperty(DIPROP_RANGE, &diPropRange.diph);
	
	if((hResult != DI_OK) && (hResult != DI_PROPNOEFFECT))
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::setProperties > (FEHLER!!!) Konnte X-Achsen Bereich nicht setzen\n");
		#endif

		return hResult;
	}

	// Daten setzen, um den Bereich der Y-Achse zu definieren	
	diPropRange.diph.dwObj = DIJOFS_Y;
	
	hResult = m_lpJoystickDevice->SetProperty(DIPROP_RANGE, &diPropRange.diph);
	
	if((hResult != DI_OK) && (hResult != DI_PROPNOEFFECT))
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::setProperties > (FEHLER!!!) Konnte Y-Achsen Bereich nicht setzen\n");
		#endif

		return hResult;
	}

	// Daten setzen, um die DeadZone der X-Achse zu definieren
	diPropDword.diph.dwSize			= sizeof(diPropDword); 
	diPropDword.diph.dwHeaderSize	= sizeof(diPropDword.diph); 
	diPropDword.diph.dwObj			= DIJOFS_X; 
	diPropDword.diph.dwHow			= DIPH_BYOFFSET; 
	diPropDword.dwData				= nDeadZone; 

	hResult = m_lpJoystickDevice->SetProperty(DIPROP_DEADZONE, &diPropDword.diph);
	
	if((hResult != DI_OK) && (hResult != DI_PROPNOEFFECT))
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::setProperties > (FEHLER!!!) Konnte X-Achsen DeadZone nicht setzen\n");
		#endif

		return hResult;
	}

	// Daten setzen, um die DeadZone der Y-Achse zu definieren
	diPropDword.diph.dwObj = DIJOFS_Y; 	
	
	hResult = m_lpJoystickDevice->SetProperty(DIPROP_DEADZONE, &diPropDword.diph);
	
	if((hResult != DI_OK) && (hResult != DI_PROPNOEFFECT))
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::setProperties > (FEHLER!!!) Konnte X-Achsen DeadZone nicht setzen\n");
		#endif

		return hResult;
	}

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::activateJoystick
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Initialisiert und aktiviert den Joystick
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::activateJoystick(HWND hWnd, int nRange, unsigned int nDeadZone)
{
	HRESULT hResult;

	//Joystick-Device anlegen
	hResult = m_lpDirectInput->CreateDeviceEx(GUID_Joystick, IID_IDirectInputDevice7, (void**)&m_lpJoystickDevice, NULL);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateJoystick > (FEHLER!!!) Konnte Device nicht anlegen\n");
		#endif
		
		// Muss das so ???
		m_lpJoystickDevice = NULL;
		return hResult;
	}

	//Joystick-Datenformat setzen
	hResult = m_lpJoystickDevice->SetDataFormat(&c_dfDIJoystick);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateJoystick > (FEHLER!!!) Konnte Datenformat nicht setzen\n");
		#endif

		return hResult;
	}

	/* Kooperativen Level fr den Joystick setzen
	   Andere Anwendungen knnen das Gert ausschlielich nicht exklusiv
	   bernehmen, ihre Anwendung hat nur Zugriff auf Eingabedaten, wenn
	   sie im Vordergrund luft	*/
	hResult = m_lpJoystickDevice->SetCooperativeLevel(hWnd, DISCL_EXCLUSIVE | DISCL_FOREGROUND);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateJoystick > (FEHLER!!!) Konnte Kooperativen Level nicht setzen\n");
		#endif

		return hResult;
	}

	// Eigenschaften setzen
	hResult = setJoystickProperties(nRange, nDeadZone);

	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateJoystick > (FEHLER!!!) Konnte Eigenschaften nicht setzen\n");
		#endif

		return hResult;
	}

	// Und zum Schluss - her mit dem Joystick
	hResult = m_lpJoystickDevice->Acquire();
	
	if((hResult != DI_OK) && (hResult != S_FALSE))
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateJoystick > (FEHLER!!!) Acquire nicht erfolgreich\n");
		#endif

		return hResult;
	}

	#if defined(XDEBUG) || defined(XINPUTDEBUG)
	XLogging::log("XInput::activateJoystick > DirectInput-Joystick-Device erzeugt\n");
	#endif

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::activateKeyboard
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Initialisiert und aktiviert die Tastatur
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::activateKeyboard(HWND hWnd)
{
	HRESULT hResult;

	//Keyboard-Device anlegen
	hResult = m_lpDirectInput->CreateDeviceEx(GUID_SysKeyboard, IID_IDirectInputDevice7, 
		      (void**)&m_lpKeyboardDevice, NULL);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateKeyboard > (FEHLER!!!) Konnte Device nicht anlegen\n");
		#endif
		
		// Muss das so ???
		m_lpKeyboardDevice = NULL;
		return hResult;
	}

	//Keyboard-Datenformat setzen
	hResult = m_lpKeyboardDevice->SetDataFormat(&c_dfDIKeyboard);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateKeyboard > (FEHLER!!!) Konnte Datenformat nicht setzen\n");
		#endif

		return hResult;
	}
	
	/* Kooperativen Level fr das Keyboard setzen
	   Andere Anwendungen knnen das Gert exklusiv oder nicht exklusiv
	   bernehmen, ihre Anwendung hat nur Zugriff auf Eingabedaten, wenn
	   die im Vordergrund luft */
	hResult = m_lpKeyboardDevice->SetCooperativeLevel(hWnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateKeyboard > (FEHLER!!!) Konnte Kooperativen Level nicht setzen\n");
		#endif

		return hResult;
	}

	// Und zum Schluss - her mit der Tastatur
	hResult = m_lpKeyboardDevice->Acquire();
	
	if((hResult != DI_OK) && (hResult != S_FALSE))
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateKeyboard > (FEHLER!!!) Acquire nicht erfolgreich\n");
		#endif

		return hResult;
	}

	#if defined(XDEBUG) || defined(XINPUTDEBUG)
	XLogging::log("XInput::activateKeyboard > DirectInput-Keyboard-Device erzeugt\n");
	#endif

	return hResult;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::getKeyboardText
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Holt Text passend zur Taste, und kopiert ihn in den char*
/////////////////////////////////////////////////////////////////////////////////////
HRESULT XInput::getKeyboardText(DWORD dwNrOfKey,char* pcText,int iMaxLength)
{
	HRESULT hResult;
	DIDEVICEOBJECTINSTANCE didoiTmp;
	int iTmp;
	
	didoiTmp.dwSize=sizeof(DIDEVICEOBJECTINSTANCE);

	hResult=m_lpKeyboardDevice->GetObjectInfo(&didoiTmp,dwNrOfKey,DIPH_BYOFFSET);

	if (hResult==DI_OK)
	{
		for (iTmp=0;iTmp+1<iMaxLength && didoiTmp.tszName[iTmp]!=0;iTmp++)
		{
			pcText[iTmp]=didoiTmp.tszName[iTmp];
		}
		pcText[iTmp]=0;
	}
	#if defined(XDEBUG) || defined(XINPUTDEBUG)
	else
		XLogging::log("XInput::getKeyboardText > (FEHLER!!!) ObjectInfo nicht erhalten!\n");
	#endif

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::activateMouse
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Initialisiert und aktiviert die Maus
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::activateMouse(HWND hWnd, int nXStartPos, int nYStartPos)
{
	DIPROPDWORD	diMouse;
	HRESULT		hResult;
	
	//Maus-Device anlegen
	hResult = m_lpDirectInput->CreateDeviceEx(GUID_SysMouse, IID_IDirectInputDevice7, (void**)&m_lpMouseDevice, NULL);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateMouse > (FEHLER!!!) Konnte Device nicht anlegen\n");
		#endif
		
		// Muss das so ???
		m_lpMouseDevice = NULL;
		return hResult;
	}

	//Maus-Datenformat setzen
	hResult = m_lpMouseDevice->SetDataFormat(&c_dfDIMouse);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateMouse > (FEHLER!!!) Konnte Datenformat nicht setzen\n");
		#endif

		return hResult;
	}
	
	/* Kooperativen Level fr die Maus setzen
	   Andere Anwendungen knnen das Gert ausschlielich nicht exklusiv
	   bernehmen, ihre Anwendung hat nur Zugriff auf Eingabedaten, wenn
	   sie im Vordergrund luft	*/
	hResult = m_lpMouseDevice->SetCooperativeLevel(hWnd, DISCL_EXCLUSIVE | DISCL_FOREGROUND);
 
	if(hResult != DI_OK) 
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateMouse > (FEHLER!!!) Konnte Kooperativen Level nicht setzen\n");
		#endif

		return hResult;
	}

	// Puffer initialisieren
	diMouse.diph.dwSize			= sizeof(DIPROPDWORD);
	diMouse.diph.dwHeaderSize	= sizeof(DIPROPHEADER);
	diMouse.diph.dwObj			= 0;
	diMouse.diph.dwHow			= DIPH_DEVICE;
	diMouse.dwData				= 1;

	// Eigenschaften setzen
	hResult = m_lpMouseDevice->SetProperty(DIPROP_BUFFERSIZE, &diMouse.diph);

	if(hResult != DI_OK)
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateMouse > (FEHLER!!!) Konnte Eigenschaften nicht setzten\n");
		#endif
		
		return hResult;
	}

	// Und zum Schluss - her mit der Maus
	hResult = m_lpMouseDevice->Acquire();

	if((hResult != DI_OK) && (hResult != S_FALSE))
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::activateMouse > (FEHLER!!!) Acquire nicht erfolgreich\n");
		#endif

		return hResult;
	}

	#if defined(XDEBUG) || defined(XINPUTDEBUG)
	XLogging::log("XInput::activateMouse > DirectInput-Mouse-Device erzeugt\n");
	#endif

	return hResult;
}


//////////////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::getJoystickData
//////////////////////////////////////////////////////////////////////////////////////////////
// Info:			Besorgt Daten vom Joystick
//////////////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::getJoystickData()
{
	HRESULT hResult;
	DIJOYSTATE diJoyState;


	hResult = m_lpJoystickDevice->Poll();

	if((hResult != DI_OK) && (hResult != DI_NOEFFECT))
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::getJoystickData > (FEHLER!!!) Poll nicht erfolgreich\n");
		#endif

		return hResult;
	}

	hResult = m_lpJoystickDevice->GetDeviceState(sizeof(diJoyState), &diJoyState);
	
	if(hResult != DI_OK)
	{
		if(hResult == DIERR_INPUTLOST)
		{
			// Sei mein Untertan - JOY Stick 
			hResult = m_lpJoystickDevice->Acquire();
			if((hResult != DI_OK) && (hResult != S_FALSE))
			{
				#if defined(XDEBUG) || defined(XINPUTDEBUG)
				XLogging::log("XInput::getJoystickData > (FEHLER!!!) Acquire nicht erfolgreich\n");
				#endif

				return hResult;
			}
			else
			{
				// Nochmal Daten holen - wenn Fehler - Arschlecken und raus
				hResult = m_lpJoystickDevice->GetDeviceState(sizeof(diJoyState), &diJoyState);

				if(hResult != DI_OK)
				{
					#if defined(XDEBUG) || defined(XINPUTDEBUG)
					XLogging::log("XInput::getJoystickData > (FEHLER!!!) GetDeviceState Failure\n");
					#endif

					return hResult;
				}
			}
		}
		else
		{
			#if defined(XDEBUG) || defined(XINPUTDEBUG)
			XLogging::log("XInput::getJoystickData > (FEHLER!!!) GetDeviceState Failure\n");
			#endif

			return hResult;
		}
	}

	m_dwJoyXPos = diJoyState.lX;
	m_dwJoyYPos = diJoyState.lY;

	for(int i = 0; i < 4; i++)
	{
		if(diJoyState.rgbButtons[i] & 0x80)
		{
			m_bJoyButtonPressed[i] = true;
		}
		else
		{
			m_bJoyButtonPressed[i] = false;
		}
	}

	// Positiver X-Bereich 
	if(m_dwJoyXPos > 0)
	{
		m_bJoyMoved[1] = true;

		m_bJoyMoved[0] = false;
		m_bJoyMoved[2] = false;
		m_bJoyMoved[3] = false;
	}

	// Negativer X-Bereich
	else if(m_dwJoyXPos < 0)
	{
		m_bJoyMoved[3] = true;

		m_bJoyMoved[0] = false;
		m_bJoyMoved[1] = false;
		m_bJoyMoved[2] = false;
	}

	// Positiver Y-Bereich
	else if(m_dwJoyYPos > 0)
	{
		m_bJoyMoved[0] = true;

		m_bJoyMoved[1] = false;
		m_bJoyMoved[2] = false;
		m_bJoyMoved[3] = false;
	}

	// Negativer Y-Bereich
	else if(m_dwJoyYPos < 0)
	{
		m_bJoyMoved[2] = true;

		m_bJoyMoved[0] = false;
		m_bJoyMoved[1] = false;
		m_bJoyMoved[3] = false;
	}
	else
	{
		m_bJoyMoved[0] = false;
		m_bJoyMoved[1] = false;
		m_bJoyMoved[2] = false;
		m_bJoyMoved[3] = false;
	}

	return hResult;
}


//////////////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::getKeyboardData
//////////////////////////////////////////////////////////////////////////////////////////////
// Info:			Besorgt Daten von der Tastatur
//////////////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::getKeyboardData()
{
	HRESULT hResult;

	// Daten holen - wenn Fehler - nochmal Acquire aufrufen	
	hResult = m_lpKeyboardDevice->GetDeviceState(256, &m_keyboardData);

	if(hResult != DI_OK)
	{
		if(hResult == DIERR_INPUTLOST)
		{
			// Sei mein Untertan - Tastenschlampe 
			hResult = m_lpKeyboardDevice->Acquire();
			if((hResult != DI_OK) && (hResult != S_FALSE))
			{
				#if defined(XDEBUG) || defined(XINPUTDEBUG)
				XLogging::log("XInput::getKeyboardData > (FEHLER!!!) Acquire nicht erfolgreich\n");
				#endif

				return hResult;
			}
			else
			{
				// Nochmal Daten holen - wenn Fehler - Arschlecken und raus
				hResult = m_lpKeyboardDevice->GetDeviceState(256, &m_keyboardData);

				if(hResult != DI_OK)
				{
					#if defined(XDEBUG) || defined(XINPUTDEBUG)
					XLogging::log("XInput::getMouseData > (FEHLER!!!) GetDeviceState Failure\n");
					#endif

					return hResult;
				}
			}
		}
		else
		{
			#if defined(XDEBUG) || defined(XINPUTDEBUG)
			XLogging::log("XInput::getKeyboardData > (FEHLER!!!) GetDeviceState Failure\n");
			#endif

			return hResult;
		}
	}

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::getMouseData
/////////////////////////////////////////////////////////////////////////////////////
// Info:			besorgt Daten von der Maus
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::getMouseData()
{
	HRESULT hResult;
	DIMOUSESTATE diMouseState;

	// Daten holen - wenn Fehler - nochmal Acquire aufrufen
	hResult = m_lpMouseDevice->GetDeviceState(sizeof(DIMOUSESTATE), &diMouseState);

	if(hResult != DI_OK)
	{
		if(hResult == DIERR_INPUTLOST)
		{
			// Sei mein Untertan - Maus 
			hResult = m_lpMouseDevice->Acquire();
			if((hResult != DI_OK) && (hResult != S_FALSE))
			{
				#if defined(XDEBUG) || defined(XINPUTDEBUG)
				XLogging::log("XInput::getMouseData > (FEHLER!!!) Acquire nicht erfolgreich\n");
				#endif

				return hResult;
			}
			else
			{
				// Nochmal Daten holen - wenn Fehler - Arschlecken und raus
				hResult = m_lpMouseDevice->GetDeviceState(sizeof(DIMOUSESTATE), &diMouseState);
				
				if(hResult != DI_OK)
				{
					#if defined(XDEBUG) || defined(XINPUTDEBUG)
					XLogging::log("XInput::getMouseData > (FEHLER!!!) GetDeviceState Failure\n");
					#endif

					return hResult;
				}
			}
		}
		else
		{
			#if defined(XDEBUG) || defined(XINPUTDEBUG)
			XLogging::log("XInput::getMouseData > (FEHLER!!!) GetDeviceState Failure\n");
			#endif

			return hResult;
		}
	}

	// Daten auslesen
	m_dwMouseXPos += diMouseState.lX;
	m_dwMouseYPos += diMouseState.lY;

	if(m_lpRect)
	{
		if(m_dwMouseXPos <= m_lpRect->left)
		{
			m_dwMouseXPos = m_lpRect->left;
		}
		if(m_dwMouseXPos >= m_lpRect->right)
		{
			m_dwMouseXPos = m_lpRect->right;
		}
		if(m_dwMouseYPos <= m_lpRect->top)
		{
			m_dwMouseYPos = m_lpRect->top;
		}
		if(m_dwMouseYPos >= m_lpRect->bottom)
		{
			m_dwMouseYPos = m_lpRect->bottom;
		}
	}

	for(int i = 0; i < 3; i++)
	{
		if(diMouseState.rgbButtons[i] & 0x80)
		{
			m_bMouseButtonPressed[i] = true;
		}
		else
		{
			m_bMouseButtonPressed[i] = false;
		}
	}

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::unacquireJoystick
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Fhrt ein explizites Unacquire auf den Joystick aus
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::unacquireJoystick()
{
	HRESULT hResult;
	
	hResult = m_lpJoystickDevice->Unacquire();
	if(hResult != DI_OK)
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::unacquireJoystick > (FEHLER!!!) Unacquire Failure\n");
		#endif

		return hResult;
	}

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::unacquireKeyboard
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Fhrt ein explizites Unacquire auf das Keyboard aus
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::unacquireKeyboard()
{
	HRESULT hResult;
	
	hResult = m_lpKeyboardDevice->Unacquire();
	if(hResult != DI_OK)
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::unacquireKeyboard > (FEHLER!!!) Unacquire Failure\n");
		#endif

		return hResult;
	}

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::unacquireMouse
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Fhrt ein explizites Unacquire auf die Maus aus
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::unacquireMouse()
{
	HRESULT hResult;

	hResult = m_lpMouseDevice->Unacquire();
	if(hResult != DI_OK)
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::unacquireMouse > (FEHLER!!!) Unacquire Failure\n");
		#endif

		return hResult;
	}

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			De-Initialisiert die DInput-(Device-)Objekte
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XInput::uninitialize()
{
	ULONG uLong;

	if(m_lpDirectInput) 
    { 
        // Maus-Device lsen
		if(m_lpMouseDevice) 
        { 
			m_lpMouseDevice->Unacquire(); 
            uLong = m_lpMouseDevice->Release();

			#if defined(XDEBUG) || defined(XINPUTDEBUG)
			XLogging::log("XInput::uninitialize > DirectInput7-Mouse-Device-Release = ");
			XLogging::log(uLong);
			XLogging::log("\n");
			#endif	
            
			m_lpMouseDevice = NULL; 
        } 

		// Keyboard-Device lsen
        if(m_lpKeyboardDevice) 
        { 
			m_lpKeyboardDevice->Unacquire(); 
            uLong = m_lpKeyboardDevice->Release();

			#if defined(XDEBUG) || defined(XINPUTDEBUG)
			XLogging::log("XInput::uninitialize > DirectInput7-Keyboard-Device-Release = ");
			XLogging::log(uLong);
			XLogging::log("\n");
			#endif	

			m_lpKeyboardDevice = NULL; 
        } 
        
		// Joystick/Joypad-Device lsen
		if(m_lpJoystickDevice)
		{
			m_lpJoystickDevice->Unacquire(); 
            uLong = m_lpJoystickDevice->Release();

			#if defined(XDEBUG) || defined(XINPUTDEBUG)
			XLogging::log("XInput::uninitialize > DirectInput7-Joystick-Device-Release = ");
			XLogging::log(uLong);
			XLogging::log("\n");
			#endif	

			m_lpJoystickDevice = NULL; 
		}

		// Hier einfgen falls weitere Devices angelegt werden

		// DirectInput-Objekt lsen
		uLong = m_lpDirectInput->Release();

		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::uninitialize > DirectInput7-Objekt-Release = ");
		XLogging::log(uLong);
		XLogging::log("\n");
		#endif	

        m_lpDirectInput = NULL; 
    } 

	return DI_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::setRect
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Initialisiert Rect-Struktur fr die Bereiche der Maus
/////////////////////////////////////////////////////////////////////////////////////

bool XInput::setRect(int left, int top, int right, int bottom)
{
	if(m_lpRect == NULL)
	{
		m_lpRect = new RECT;
		
		m_lpRect->top		= top;
		m_lpRect->left		= left;
		m_lpRect->bottom	= bottom - 1;
		m_lpRect->right		= right - 1;
	}
	else
	{
		m_lpRect->top		= top;
		m_lpRect->left		= left;
		m_lpRect->bottom	= bottom - 1;
		m_lpRect->right		= right - 1;
	}
	
	return true;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XInput::releaseRect
/////////////////////////////////////////////////////////////////////////////////////
// Info:			De-Initialisiert Rect-Struktur
/////////////////////////////////////////////////////////////////////////////////////

bool XInput::releaseRect()
{
	if(m_lpRect)
	{
		delete m_lpRect;
		m_lpRect = NULL;
		
		return true;
	}
	else
	{
		#if defined(XDEBUG) || defined(XINPUTDEBUG)
		XLogging::log("XInput::setRect > (WARNUNG!!!) Rect-Struktur existiert nicht\n");
		#endif

		return false;
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XInput.cpp
/////////////////////////////////////////////////////////////////////////////////////