/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XTimer.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	24.07.2001
// letzte nderung:	31.07.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Diese Klasse ermglicht millisekunden genaues Timing, intern wird		
//			allerdings mit der hchstmglichen Auflsung (evtl. PerformanceTimer)
//			gearbeitet
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "XTimer.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XTimer::XTimer
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Konstruktor
/////////////////////////////////////////////////////////////////////////////////////
XTimer::XTimer()
{

};

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XTimer::setUp
/////////////////////////////////////////////////////////////////////////////////////
// Info:	initialisiert alle Member der Klasse
//			findet Timerfrequenz heraus
/////////////////////////////////////////////////////////////////////////////////////
void XTimer::setUp(void)
{
	__int64	liFrequency;
	BOOL	bRes;

	m_dwTimeInMS=0;
	m_dwOverflow=0;

	bRes=QueryPerformanceFrequency((LARGE_INTEGER*)&liFrequency);
	
	if (bRes!=0)
	{
		#if defined(XDEBUG) || defined(XTIMERDEBUG)
		XLogging::log("XTimer::XTimer > initializing PerformanceCounter\n");
		#endif

		m_bPerformaceTimerEnabled=true;
		m_fTicksPerMS=(float)liFrequency/1000.0f;
		QueryPerformanceCounter((LARGE_INTEGER*)&m_liLastTime);
	}
	else
	{
		#if defined(XDEBUG) || defined(XTIMERDEBUG)
		XLogging::log("XTimer::XTimer > initializing GetTickCount\n");
		#endif

		m_bPerformaceTimerEnabled=false;

		m_fTicksPerMS=1.0f;
		m_liLastTime=(__int64)GetTickCount();

	}
};

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XTimer::setTime
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt den ms-Zhler auf die gewnschte Zeit
////////////////////////////////////////////////////////////////////////////////////
void XTimer::setTime(DWORD newTime)
{
	m_dwTimeInMS=newTime;
	m_dwOverflow=0;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XTimer::getTime
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt die aktuelle Zeit des ms-Zhlers beim letzten Update()
/////////////////////////////////////////////////////////////////////////////////////
DWORD XTimer::getTime()
{
	return m_dwTimeInMS;	
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XTimer::update
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt zurck wieviel Zeit seit dem letzten Update vergangen ist (in ms)
/////////////////////////////////////////////////////////////////////////////////////
DWORD XTimer::update()
{
	__int64	liNewTime;
	DWORD	dwTicksPassed;
	DWORD	dwMSPassed;

	if (m_bPerformaceTimerEnabled)
	{
		QueryPerformanceCounter((LARGE_INTEGER*)&liNewTime);
	}
	else
	{
		liNewTime=GetTickCount();
	}

	dwTicksPassed=(DWORD)(liNewTime - m_liLastTime)+m_dwOverflow;

	dwMSPassed=(DWORD)((float)dwTicksPassed /m_fTicksPerMS);

	m_dwOverflow=dwTicksPassed-(DWORD)(dwMSPassed*m_fTicksPerMS);

	m_dwTimeInMS+=dwMSPassed;

	m_liLastTime=liNewTime;

	return dwMSPassed;
}


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XTimer.cpp
/////////////////////////////////////////////////////////////////////////////////////
